package installchecker;

use strict;

my $CVS = '$Id: installchecker.pm,v 1.1 2003/12/26 08:10:42 othauvin Exp $';

use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw/parse_rpmsrate parse_compssUsers/;

sub report_error {
	my ($type, $msg);
	($type, $msg, @_) = @_;
	$type =~ /^[WE]$/ or die "Wrong use of report_error, \$type = $type $msg\n";
	printf("$type\: $msg\n",@_);
}

sub parse_rpmsrate {
	my ($file, %options) = @_;
	my $pkgs;
	
	$options{report_error} ||= \&report_error;
	
	open RPMSRATE, $file or die "Can't open rpmsrate '$file'";
	
	my $nb_line;
	my @l;
	# @group -> @HASHREF { 'length' => nb of space, name => group name }
	my @group;
	
	# $package = { 
	# 	@want_or => list of group wanted (or)
	# 	$level => level to set
	my $package;

	while (<RPMSRATE>) {
		$nb_line++;
		# No tab in this file
		/\t/ and $options{report_error}('E', "rpmsrate <Tab> not allow at line %d", $nb_line); 
		s/#.*//; # comments
		my ($indent, $data) = /(\s*)(.*)/;
		next if !$data; # skip empty lines
		
		length($indent) % 2 == 0 or $options{report_error}('E', "rpmsrate indent is not <space> * 2 at line %d", $nb_line);
		
		@l = grep { $_->[0] < length $indent } @l;
		my $this_group = @group ? $group[-1] : []; # only the latest
		# keep only if ident is bigger
		@$this_group = grep { $_->{'length'} < length $indent } @$this_group;
		
		my @pack_param = @l ? @{$l[-1][1]} : ();
		

		# Find flag and level
        	my ($t, $flag, @l2);
		while ($data =~
               		/^((
                   		[1-5]
	                   	|
        	           	(?:            (?: !\s*)? [0-9A-Z_]+(?:".*?")?)
                	   	(?: \s*\|\|\s* (?: !\s*)? [0-9A-Z_]+(?:".*?")?)*
                	  )
                  	(?:\s+|$)
                 	)(.*)/x) { #@")) {
	            ($t, $flag, $data) = ($1,$2,$3);
		    $flag =~ /^\d$/ and do {
			    $package->{level} = $flag;
			    next;
		    };
		    
        	    while ($flag =~ s,^\s*(("[^"]*"|[^"\s]*)*)\s+,$1,) {}
		    if (0) {
            		my $ok = 0;
            		$flag = join('||', grep {
                		if (my ($inv, $p) = /^(!)?HW"(.*)"/) {
					#($inv xor detect_devices::matching_desc($p)) and $ok = 1;
                    				0;
                			} else {
                    				1;
                			}
            		} split '\|\|', $flag);
		
            		push @pack_param, $ok ? 'TRUE' : $flag || 'FALSE';
			#$options{report_error}(10, "m is %s", join(' ',@pack_param));
            		push @l2, [ length $indent, [ @pack_param ] ];
            		$indent .= $t;
		    }
		    }
		$flag and do {
			$flag =~ s/^\d//;
			length $flag and do {
				push(@$this_group, { 'length' => length $indent, name => $flag });
			};
		};
		# Here data should only contain package
		if ($data) {
            	# has packages on same line
			#my $rate=0;
			#$options{report_error}(10,"param for package are %s",join(',',@pack_param));
			#my $rate = find { /^\d$/ } @m or $options{report_error}(0, qq(missing rate for "%s" at line %d (flags are %s)\n), $data, $nb_line, join('&&', @m));
            		foreach (split ' ', $data) {
				if ($pkgs->{$_}{level}) {
					$pkgs->{$_}{level} != $package->{level} and $options{report_error}('E',"Can't handle different rate for %s (%d != %d)", $_, $pkgs->{$_}{level}, $package->{level});
				} else {
					$pkgs->{$_}{level} = $package->{level};
				}
				push (@{$pkgs->{$_}{want}}, join (" && ", map { $_->{name} } @$this_group));
				#print join (" && ", map { $_->{name} } @$this_group);
				#print " $pkgs->{$_}{level}\n";
				
			#if ($pkgs) {
				#my $p = packageByName($pkgs, $_) or next;
				#my @m2 = map { if_(/locales-(.*)/, qq(LOCALES"$1")) } $p->requires_nosense;
                    		#my @m3 = ((grep { !/^\d$/ } @m), @m2);
				#if (member('INSTALL', @m3)) {
				#member('NOCOPY', @m3) or push @{$pkgs->{needToCopy} ||= []}, $_;
				#next; #- don't need to put INSTALL flag for a package.
			#}
			#if ($p->rate) {
			#my @m4 = $p->rflags;
			#	if (@m3 > 1 || @m4 > 1) {
			#    	$options{report_error}(0,"can't handle complicate flags for packages appearing twice ($_)");
			#}
			#$options{report_error}("package $_ appearing twice with different rates ($rate != ".$p->rate.")")
			#if $rate != $p->rate;
 			#$p->set_rate($rate);
			#$p->set_rflags("$m3[0]||$m4[0]");
			#} else {
			   #$p->set_rate($rate);
			   #$p->set_rflags(@m3);
			   #		}
			   #	} else {
			   #		print "$_ = ", join(" && ", @m), "\n";
			   #	}
			   #}
			#push @l, @l2;
				#} else {
				#push @l, [ $l2[0][0], $l2[-1][1] ];
			
			}
			push(@group, $this_group);
    		}

		
	}
	close RPMSRATE;
	$pkgs
}

sub parse_compssUsers {
	my ($file, %options) = @_;
	my $groups;
	
	$options{report_error} ||= \&report_error;
	
	# %$compss_entry =
	# 	path
	# 	icon
	# 	descr
	my $nb_line;
	open COMPSSUSERS, $file or die "Can't open compssUsers '$file'\n";
	while (<COMPSSUSERS>) {
		$nb_line++;
		s/\#.*$//;
		/\S/ or next;
		if (/^\S/) {
			my $compss_group = {};
			/^(.*?)\[/ and $compss_group->{name}=$1;
			/^.*\[icon=(.*?)\].*$/ and $compss_group->{icon}=$1;
			/^.*\[path=(.*?)\].*$/ and $compss_group->{path}=$1;
			/^.*\[descr=(.*?)\].*$/ and $compss_group->{descr}=$1;
			
			push @$groups, $compss_group; 	
			
		} else {
			/^\s*(.*?)\s*$/;
			$1 or next;
			!@$groups and $options{report_error}('E', "No group defined in compssUser line %d", $nb_line);
			push @{$groups->[-1]{wanted}}, $1;
		}
	}
	close COMPSSUSERS;
	$groups
}

