package urpmchecker;

# $Id: query.pm,v 1.16 2003/10/04 03:59:20 tvignaud Exp $

=begin

=head1 NAME

urpmchecker::query - function to query rpm from urpmchecker

=head1 SYMOPSYS

use urpmchecker;
use urpmchecker::query;
$urpmchecker = new urpmchecker;

	
=head1 SEE ALSO

perl-URPM
urpmchecker

=head1 LICENSE

Olivier Thauvin <thauvin@aerov.jussieu.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

=head1 DESCRIPTION

C<urpmchecker> include urpmchecker functions

=cut

use strict;
use URPM;
use URPM::Build;
use BerkeleyDB;
use MDK::Common;

=item * $urpmchecker->pgkname($id);

return name-version-release.arch (rpm source) [urpmchecker_level]

=cut

sub pkgname {
    my ($self, $u_rpm) = @_;
    my $pkg = $self->{urpm}{depslist}[$u_rpm] or return;
    return sprintf("%s (%s) [%s]", scalar $pkg->fullname, $pkg->arch ne 'src' ? $pkg->sourcerpm : '', $self->{id_level}{$u_rpm});
}

sub rpmqi {
    my ($self, $u_rpm) = @_;
    my $pkg=$self->{urpm}{depslist}[$u_rpm] or return;
    return
        sprintf("Name     : %s [%s]\n", scalar $pkg->fullname, $self->{id_level}{$u_rpm}) .
        sprintf("Source   : %s\n", $pkg->sourcerpm) .
        sprintf("Group    : %-20s\n", $pkg->group) .
        sprintf("Packager : %-20s\n", $pkg->packager) .
        sprintf("Url      : %s\n", $pkg->url) .
        sprintf("Summary  : %s\n", $pkg->summary) .
        ($pkg->buildarchs ? sprintf("BuildArch: %s\n", $pkg->buildarchs) : '') .
        ($pkg->exclusivearchs ? sprintf("ExclusiveArch: %s\n", join(',', $pkg->exclusivearchs)) : '') .
        ($pkg->excludearchs ? sprintf("ExcludeArch: %s\n", join(',', $pkg->excludearchs)) : '');
}

sub rpmql {
    my ($self, $u_rpm) = @_;
    my $pkg = $self->{urpm}{depslist}[$u_rpm] or return;
    my @f = $pkg->files;
    my @m = $pkg->files_mode;
	my @u = $pkg->files_owner;
	my @g = $pkg->files_group;
    map { $self->human_files($f[$_], undef, $m[$_], undef, $u[$_], $g[$_]) } 0 .. $#f;
}

sub human_files {
    my ($_self, $file, $_u_rpm, $mode, $_md5sum, $user, $group)=@_;
    my $e;
    ($mode & 0170000) == 0040000 and $e .= 'd';
    ($mode & 0170000) == 0020000 and $e .= 'c';
    ($mode & 0170000) == 0060000 and $e .= 'b';
    ($mode & 0170000) == 0100000 and $e .= '-';
    ($mode & 0170000) == 0010000 and $e .= 'p';
    ($mode & 0170000) == 0120000 and $e .= 'l';
    ($mode & 0170000) == 0140000 and $e .= 's';
    my $dec = $mode & 0000777;
    my $setuid = $mode & 0007000;
    $setuid = $setuid << 1; 
    foreach (0 .. 2) {
        $e .= $dec & 0000400 ? 'r' : '-';
        $e .= $dec & 0000200 ? 'w' : '-';
        $e .= $setuid & 0010000 
            ? ($mode & 0170000) == 0040000 ? 't' : 's' 
            : $dec & 0000100 ? 'x' : '-';
        $dec = $dec << 3;
        $setuid = $setuid << 1; 
    }
    sprintf("%s   %-10s %-10s %s", $e, $user, $group, $file)
}

sub rpmreq {
    my ($self, $u_rpm) = @_;
    $self->{urpm}{depslist}[$u_rpm] or next;
    my @l;
    foreach my $u_rpmd (keys %{$self->{deps}}) {
        foreach my $dep (@{$self->{deps}{$u_rpmd}}) {
            $dep =~ /^NOT/ and next;
            any { $u_rpm eq $_ } split('\|', $dep) and push @l, $u_rpmd;
        }
    }
    uniq(@l)
}

sub rpmdep {
   my ($self, $u_rpm) = @_;
   my @ret;
   $self->{urpm}{depslist}[$u_rpm] or next;
   my @dep = $self->{urpm}{depslist}[$u_rpm]->requires;
   foreach (@{$self->{deps}{$u_rpm}}) {
        my $d = shift(@dep);
        if (/^NOT/) {
            push @ret, "$d: Not found";
        } else {
            push @ret, "$d: ".join('|', map { $self->pkgname($_) } split('\|'));
        }
    }
    @ret
}

sub find_file {
    my ($self, $regexp) = @_;
    $regexp =~ s!/!\\/!g;
    my $cursor = $self->{dbfiles}->db_cursor;
    my ($file, $value, %res);
    while ($cursor->c_get($file, $value, DB_NEXT) == 0) {
        eval("\$file =~ /$regexp/") or next;
        while ($cursor->c_get($file, $value, DB_NEXT_DUP) == 0) {
            my ($name, @v) = split(' ', $value);
            push @{$res{$name}}, $self->human_files($file, $name, @v);
        }
    }
    %res
}

sub try_move {
    my ($self, @rpms) = @_;
    my @break;
    my $level = pop(@rpms);
    
    foreach my $rpm (@rpms) {
        foreach my $dep ($self->pkg_need($rpm)) {
            any { $_ eq $dep } @rpms and next;
            $self->{id_level}{$dep} > $level and push(@break,$dep);
        }

        foreach my $dep ($self->rpmreq($rpm)) {
            any { $_ eq $dep } @rpms and next;
            $self->{id_level}{$dep} < $level and push(@break,$dep);
        }
    }
    uniq(@break)
}

1;
